//@ts-check
"use strict";

/**
@type {{
  width: number; height: number;
  stageScale: number;
  searchParams: URLSearchParams;
  screenPixiRoot: PIXI.Container;
  language: string;
  cursor: { pos: PIXI.Point; sprite: PIXI.AnimatedSprite; spritesheet: Spritesheet; };
  fitScreen: boolean;
}}
*/
const game = {
  width: 640,
  height: 360,
  stageScale: 1,
  searchParams: new URLSearchParams(location.search),
  screenPixiRoot: null,
  language: "en",
  cursor: { pos: new PIXI.Point(), sprite: null, spritesheet: null },
  fitScreen: localStorage.getItem("woodpunk:fitScreen") === "true"
};

PIXI.settings.SCALE_MODE = PIXI.SCALE_MODES.NEAREST;
PIXI.settings.ROUND_PIXELS = true;

const app = new PIXI.Application({ width: game.width, height: game.height, antialias: false, autoDensity: true, resolution: window.devicePixelRatio || 1, backgroundColor: 0xff000000 });
document.body.appendChild(app.view);

app.loader.baseUrl = "assets/";
app.loader.add(["cursor.png", "cursor.sprite.json"]);
registerTextResources();
registerAudioResources();
registerIntroResources();
registerMainMenuResources();
registerGameOverResources();
registerWinResources();
registerInGameResources();

/** @param {boolean} fit */
function setFitScreen(fit) {
  game.fitScreen = fit;
  localStorage.setItem("woodpunk:fitScreen", JSON.stringify(game.fitScreen));
  resize();
}

function resize() {
  app.renderer.resolution = window.devicePixelRatio || 1;

  let i = 1;
  let increment = game.fitScreen ? 0.1 : 1;
  while (
    game.width * (i + increment) < window.innerWidth * app.renderer.resolution &&
    game.height * (i + increment) < window.innerHeight * app.renderer.resolution)
    i += increment;

  const renderWidth = game.width * i / app.renderer.resolution;
  const renderHeight = game.height * i / app.renderer.resolution;

  app.renderer.resize(renderWidth, renderHeight);
  app.renderer.view.style.width = `${renderWidth}px`;
  app.renderer.view.style.height = `${renderHeight}px`;
}

resize();
window.addEventListener("resize", resize);
document.addEventListener("contextmenu", (event) => event.preventDefault());
setupDebugLog();

document.querySelector(".loading .category").textContent = "art and data";
app.loader.onProgress.add(() => document.querySelector(".loading .progress").textContent = Math.floor(app.loader.progress).toString() + "%");
app.loader.load(() => {
  registerAudioSounds();
  document.querySelector(".loading .category").textContent = "sounds";
  document.querySelector(".loading .progress").textContent = "";

  app.loader.load(() => {
    document.querySelector(".loading").remove();

    let language = localStorage.getItem("woodpunk:language");
    if (language == null) language = navigator.languages[0].split("-")[0] === "fr" ? "fr" : "en";
    setTextLanguage(language);

    game.screenPixiRoot = new PIXI.Container();
    app.stage.addChild(game.screenPixiRoot);

    game.cursor.spritesheet = getSpritesheet("cursor");
    game.cursor.sprite = new PIXI.AnimatedSprite(game.cursor.spritesheet.animations["default"]);
    app.stage.addChild(game.cursor.sprite);

    app.ticker.add((dt) => {
      game.stageScale = app.renderer.height / game.height / app.renderer.resolution;
      app.stage.scale.set(game.stageScale);
    });

    document.addEventListener("pointermove", (event) => {
      const canvasRect = app.renderer.view.getBoundingClientRect();

      game.cursor.pos.set(
        (event.clientX - canvasRect.left) / game.stageScale,
        (event.clientY - canvasRect.top) / game.stageScale);
      game.cursor.sprite.position.copyFrom(game.cursor.pos);
    });

    //@ts-ignore
    document.fonts.ready.then(() => {
      // Uncomment to change entry point:
      if (game.searchParams.has("spawn") || game.searchParams.has("world")) enterInGame();
      else enterMainMenu();
      // enterGameOver();
    });
  });
});

document.addEventListener("keydown", (event) => {
  switch (event.code) {
    case "KeyP": case "KeyF": setFitScreen(!game.fitScreen); break;
  }
});

function setGameCursor(name) {
  setAnimation(game.cursor.sprite, game.cursor.spritesheet.animations[name]);
}